/*
Arduino Neato XV-11 Motor control board v0.2 by Cheng-Lung Lee
Change log:
V0.3 Add Fuzzy logic motor speed control
V0.2 Add simple speed control code update PWM 3 times per rev.
V0.1 Opend loop control version.

This code is tested on Arduino Mega 1280
I/O:
Motor drive by low side driver IPS041L connect to PWM Pin4, Motor power from 5V
Neato XV-11 LDS Vcc(red) : 5V
Neato XV-11 LDS TX(Orange) : RX3
 
 */
#include "math.h"
#include "FuzzyFB.h"


FuzzyFB FUZZ;


int     inputs[2]={0,0};
int     outputs[1] = {0};

int	n_input_mfs[2] = { 5, 5 };

struct	In	NInputs[] =
{
	{ -800.000000, 800.000000 },
	{ -200.000000, 200.000000 }
};

float	Ninmem_points[2][7][4] =
{
	{
		{ -800.000000, -800.000000, -300.000000, -150.000000 },
		{ -300.000000, -150.000000, -150.000000, 0.000000 },
		{ -100.000000, 0.000000, 0.000000, 100.000000 },
		{ 0.000000, 150.000000, 150.000000, 300.000000 },
		{ 150.000000, 300.000000, 800.000000, 800.000000 }
	},
	{
		{ -200.000000, -200.000000, -200.000000, -66.000000 },
		{ -132.000000, -66.000000, -66.000000, 0.000000 },
		{ -66.000000, 0.000000, 0.000000, 66.000000 },
		{ 0.000000, 66.000000, 66.000000, 132.000000 },
		{ 66.000000, 132.000000, 200.000000, 200.000000 }
	}
};

int	n_output_mfs[1] = { 5 };

struct	Out	NOutputs[] =
{
	{ -2.000000, 2.000000 }
};

float	Noutmem_points[1][7][4] =
{
	{
		{ -2.000000 },
		{ -1.000000 },
		{ 0.000000 },
		{ 1.000000 },
		{ 2.000000 }
	}
};


int	n_rule_ants[25] = { 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2};
int	n_rule_cons[25] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1};


struct	Rule	NRules[25] =
{
	{ { 0x00, 0x21 }, { 0x90 } },
	{ { 0x08, 0x21 }, { 0x98 } },
	{ { 0x10, 0x21 }, { 0x98 } },
	{ { 0x18, 0x21 }, { 0xa0 } },
	{ { 0x20, 0x21 }, { 0xa0 } },
	{ { 0x00, 0x19 }, { 0x88 } },
	{ { 0x08, 0x19 }, { 0x90 } },
	{ { 0x10, 0x19 }, { 0x98 } },
	{ { 0x18, 0x19 }, { 0x98 } },
	{ { 0x20, 0x19 }, { 0xa0 } },
	{ { 0x00, 0x11 }, { 0x80 } },
	{ { 0x08, 0x11 }, { 0x88 } },
	{ { 0x10, 0x11 }, { 0x90 } },
	{ { 0x18, 0x11 }, { 0x98 } },
	{ { 0x20, 0x11 }, { 0xa0 } },
	{ { 0x00, 0x09 }, { 0x80 } },
	{ { 0x08, 0x09 }, { 0x88 } },
	{ { 0x10, 0x09 }, { 0x88 } },
	{ { 0x18, 0x09 }, { 0x90 } },
	{ { 0x20, 0x09 }, { 0x98 } },
	{ { 0x00, 0x01 }, { 0x80 } },
	{ { 0x08, 0x01 }, { 0x80 } },
	{ { 0x10, 0x01 }, { 0x88 } },
	{ { 0x18, 0x01 }, { 0x88 } },
	{ { 0x20, 0x01 }, { 0x90 } }
};

const int DesiredRPM=300;  // Setting Desired RPM Here.
const int MotorPWMPin=4;
int inByte = 0;         // incoming serial byte
unsigned char Data_status=0;
unsigned char Data_4deg_index=0;
unsigned char Data_loop_index=0;
unsigned char SpeedRPHhighbyte=0; // 
unsigned char SpeedRPHLowbyte=0;

int SpeedRPH=0;
const unsigned char PWM4dutyMax=200;
const unsigned char PWM4dutyMin=160;
unsigned char PWM4duty=170;  // have to set a default value make motor start spining

 void setup() {
   FUZZ.num_inputs = 2;
  FUZZ.num_outputs = 1;
  FUZZ.num_rules = 25;
  FUZZ.Outputs = (Out*)NOutputs;
  FUZZ.num_output_mfs =(int*) n_output_mfs;
  FUZZ.num_input_mfs = (int*)n_input_mfs;
  FUZZ.Inputs = (In*)NInputs;
  FUZZ.Rules = (Rule*) NRules;
  FUZZ.num_rule_ants = (int*)n_rule_ants ;
  FUZZ.num_rule_cons =(int*) n_rule_cons; 
  FUZZ.outmem_points = Noutmem_points;
  FUZZ.inmem_points = Ninmem_points;  
 
    pinMode(MotorPWMPin, OUTPUT); 
    Serial.begin(115200);  // USB serial
    Serial3.begin(115200);  // XV-11 LDS data 

  // prints title with ending line break 
 // Serial.println("Arduino Neato XV-11 Motor control board v0.1 by Cheng-Lung Lee"); 
  
  // Pick your magic number and drive your motor , 178 is 178/255*5V=3.49V
    analogWrite(MotorPWMPin, PWM4duty );  
}

void loop() {
    // if we get a valid byte from LDS, read it and send it to USB-serial
  
  if (Serial3.available() > 0) {
    // get incoming byte:
    inByte = Serial3.read();
    Serial.write(inByte);
    decodeData(inByte);
  }

}


void decodeData(unsigned char inByte){
  switch (Data_status){
  case 0: // no header
  if (inByte==0xFA)
  {
    Data_status=1;
    Data_loop_index=1;
 
  }
    break;
  case 1: // Find 2nd FA
    if (Data_loop_index==22){
      if (inByte==0xFA)
      {
        Data_status=2;
        Data_loop_index=1;
      } 
      else // if not FA search again
      Data_status=0;
    }
    else{
     // datas[Data_loop_index]=inByte;//dump trame
      Data_loop_index++;
      
    }
    break;
  case 2: // Read data out
  
     if (Data_loop_index==22){
  
        
      if (inByte==0xFA)
      {
        Data_loop_index=1;
      } 
      else // if not FA search again
      Data_status=0;
    }
    else{
      readData(inByte);
      Data_loop_index++;
    }
    break;
  }
  
}

int vitesse=0;
int v1=0;
int diff;

void readData(unsigned char inByte){
  switch (Data_loop_index){
    case 1: // 4 degree index
    Data_4deg_index=inByte-0xA0;

    break;
    case 2: // Speed in RPH low byte
    SpeedRPHLowbyte=inByte;
    break;
    case 3: // Speed in RPH high byte
    SpeedRPHhighbyte=inByte;
    SpeedRPH=(SpeedRPHhighbyte<<8)|SpeedRPHLowbyte;
   
    if (v1==0){
      if (SpeedRPH>13000){
       diff=18000-SpeedRPH;
       v1=1;
       }
     }
    else 
     if (SpeedRPH>13000){
     
       if (vitesse==0) {
         inputs[0]=18000-SpeedRPH;
         inputs[1]=inputs[0]-diff;
         diff=18000-SpeedRPH;
         FUZZ.fuzzy_step(inputs, outputs);
         if (outputs[0]!=0){
             vitesse=15;        
         
          if ((PWM4duty<PWM4dutyMax) &&(PWM4duty>PWM4dutyMin))
            PWM4duty+=outputs[0]; // limit the max PWM make sure it don't overflow and make LDS stop workin
          else
            PWM4duty=170;
           analogWrite(MotorPWMPin, PWM4duty );
         }
       }else
        vitesse--;
    }
    break;
    default: // others do checksum
        break;
  }  
}

